@@ -32,6 +32,7 @@
  */
 
 #include "mountP.h"
+#include "fileutils.h"
 
 #include <sys/wait.h>
 
@@ -61,14 +62,9 @@ struct libmnt_context *mnt_new_context(void)
 	/* if we're really root and aren't running setuid */
 	cxt->restricted = (uid_t) 0 == ruid && ruid == euid ? 0 : 1;
 
-	DBG(CXT, mnt_debug_h(cxt, "----> allocate %s",
+	DBG(CXT, ul_debugobj(cxt, "----> allocate %s",
 				cxt->restricted ? "[RESTRICTED]" : ""));
 
-	mnt_has_regular_mtab(&cxt->mtab_path, &cxt->mtab_writable);
-
-	if (!cxt->mtab_writable)
-		/* use /run/mount/utab if /etc/mtab is useless */
-		mnt_has_regular_utab(&cxt->utab_path, &cxt->utab_writable);
 
 	return cxt;
 }
@@ -98,7 +94,7 @@ void mnt_free_context(struct libmnt_context *cxt)
 
 	free(cxt->children);
 
-	DBG(CXT, mnt_debug_h(cxt, "<---- free"));
+	DBG(CXT, ul_debugobj(cxt, "<---- free"));
 	free(cxt);
 }
 
@@ -129,19 +125,21 @@ int mnt_reset_context(struct libmnt_context *cxt)
 	if (!cxt)
 		return -EINVAL;
 
-	DBG(CXT, mnt_debug_h(cxt, "<---- reset [status=%d] ---->",
+	DBG(CXT, ul_debugobj(cxt, "<---- reset [status=%d] ---->",
 				mnt_context_get_status(cxt)));
 
 	fl = cxt->flags;
 
 	mnt_unref_fs(cxt->fs);
 	mnt_unref_table(cxt->mtab);
+	mnt_unref_table(cxt->utab);
 
 	free(cxt->helper);
 	free(cxt->orig_user);
 
 	cxt->fs = NULL;
 	cxt->mtab = NULL;
+	cxt->utab = NULL;
 	cxt->helper = NULL;
 	cxt->orig_user = NULL;
 	cxt->mountflags = 0;
@@ -158,7 +156,9 @@ int mnt_reset_context(struct libmnt_context *cxt)
 	}
 
 	mnt_context_reset_status(cxt);
-	mnt_context_set_tabfilter(cxt, NULL, NULL);
+
+	if (cxt->table_fltrcb)
+		mnt_context_set_tabfilter(cxt, NULL, NULL);
 
 	/* restore non-resettable flags */
 	cxt->flags |= (fl & MNT_FL_NOMTAB);
@@ -173,6 +173,7 @@ int mnt_reset_context(struct libmnt_context *cxt)
 	cxt->flags |= (fl & MNT_FL_NOCANONICALIZE);
 	cxt->flags |= (fl & MNT_FL_RDONLY_UMOUNT);
 	cxt->flags |= (fl & MNT_FL_NOSWAPMATCH);
+	cxt->flags |= (fl & MNT_FL_TABPATHS_CHECKED);
 	return 0;
 }
 
@@ -200,16 +201,69 @@ int mnt_context_reset_status(struct libmnt_context *cxt)
 	return 0;
 }
 
+static int context_init_paths(struct libmnt_context *cxt, int writable)
+{
+	assert(cxt);
+
+	if (!cxt->mtab_path)
+		cxt->mtab_path = mnt_get_mtab_path();
+	if (!cxt->utab_path)
+		cxt->utab_path = mnt_get_utab_path();
+
+	if (!writable)
+		return 0;		/* only paths wanted */
+	if (mnt_context_is_nomtab(cxt))
+		return 0;		/* write mode overrided by mount -n */
+	if (cxt->flags & MNT_FL_TABPATHS_CHECKED)
+		return 0;
+
+	DBG(CXT, ul_debugobj(cxt, "checking for writable tab files"));
+
+	mnt_has_regular_mtab(&cxt->mtab_path, &cxt->mtab_writable);
+
+	if (!cxt->mtab_writable)
+		/* use /run/mount/utab if /etc/mtab is useless */
+		mnt_has_regular_utab(&cxt->utab_path, &cxt->utab_writable);
+
+	cxt->flags |= MNT_FL_TABPATHS_CHECKED;
+	return 0;
+}
+
+int mnt_context_mtab_writable(struct libmnt_context *cxt)
+{
+	assert(cxt);
+
+	context_init_paths(cxt, 1);
+	return cxt->mtab_writable == 1;
+}
+
+int mnt_context_utab_writable(struct libmnt_context *cxt)
+{
+	assert(cxt);
+
+	context_init_paths(cxt, 1);
+	return cxt->utab_writable == 1;
+}
+
+const char *mnt_context_get_writable_tabpath(struct libmnt_context *cxt)
+{
+	assert(cxt);
+
+	context_init_paths(cxt, 1);
+	return cxt->mtab_writable ? cxt->mtab_path : cxt->utab_path;
+}
+
+
 static int set_flag(struct libmnt_context *cxt, int flag, int enable)
 {
 	assert(cxt);
 	if (!cxt)
 		return -EINVAL;
 	if (enable) {
-		DBG(CXT, mnt_debug_h(cxt, "enabling flag %04x", flag));
+		DBG(CXT, ul_debugobj(cxt, "enabling flag %04x", flag));
 		cxt->flags |= flag;
 	} else {
-		DBG(CXT, mnt_debug_h(cxt, "disabling flag %04x", flag));
+		DBG(CXT, ul_debugobj(cxt, "disabling flag %04x", flag));
 		cxt->flags &= ~flag;
 	}
 	return 0;
@@ -973,6 +1027,8 @@ int mnt_context_get_mtab(struct libmnt_context *cxt, struct libmnt_table **tb)
 	if (!cxt->mtab) {
 		int rc;
 
+		context_init_paths(cxt, 0);
+
 		cxt->mtab = mnt_new_table();
 		if (!cxt->mtab)
 			return -ENOMEM;
@@ -985,7 +1041,12 @@ int mnt_context_get_mtab(struct libmnt_context *cxt, struct libmnt_table **tb)
 					cxt->table_fltrcb_data);
 
 		mnt_table_set_cache(cxt->mtab, mnt_context_get_cache(cxt));
-		rc = mnt_table_parse_mtab(cxt->mtab, cxt->mtab_path);
+		if (cxt->utab)
+			/* utab already parsed, don't parse it again */
+			rc = __mnt_table_parse_mtab(cxt->mtab,
+						    cxt->mtab_path, cxt->utab);
+		else
+			rc = mnt_table_parse_mtab(cxt->mtab, cxt->mtab_path);
 		if (rc)
 			return rc;
 	}
@@ -993,7 +1054,7 @@ int mnt_context_get_mtab(struct libmnt_context *cxt, struct libmnt_table **tb)
 	if (tb)
 		*tb = cxt->mtab;
 
-	DBG(CXT, mnt_debug_h(cxt, "mtab requested [nents=%d]",
+	DBG(CXT, ul_debugobj(cxt, "mtab requested [nents=%d]",
 				mnt_table_get_nents(cxt->mtab)));
 	return 0;
 }
@@ -1018,7 +1079,7 @@ int mnt_context_set_tabfilter(struct libmnt_context *cxt,
 				cxt->table_fltrcb,
 				cxt->table_fltrcb_data);
 
-	DBG(CXT, mnt_debug_h(cxt, "tabfiler %s", fltr ? "ENABLED!" : "disabled"));
+	DBG(CXT, ul_debugobj(cxt, "tabfilter %s", fltr ? "ENABLED!" : "disabled"));
 	return 0;
 }
 
@@ -1210,8 +1271,8 @@ struct libmnt_lock *mnt_context_get_lock(struct libmnt_context *cxt)
 		return NULL;
 
 	if (!cxt->lock) {
-		cxt->lock = mnt_new_lock(cxt->mtab_writable ?
-				cxt->mtab_path : cxt->utab_path, 0);
+		cxt->lock = mnt_new_lock(
+				mnt_context_get_writable_tabpath(cxt), 0);
 		if (cxt->lock)
 			mnt_lock_block_signals(cxt->lock, TRUE);
 	}
@@ -1390,7 +1451,7 @@ int mnt_context_prepare_srcpath(struct libmnt_context *cxt)
 	if (!cxt || !cxt->fs)
 		return -EINVAL;
 
-	DBG(CXT, mnt_debug_h(cxt, "preparing source path"));
+	DBG(CXT, ul_debugobj(cxt, "preparing source path"));
 
 	src = mnt_fs_get_source(cxt->fs);
 
@@ -1404,7 +1465,7 @@ int mnt_context_prepare_srcpath(struct libmnt_context *cxt)
 	if (!src || mnt_fs_is_netfs(cxt->fs))
 		return 0;
 
-	DBG(CXT, mnt_debug_h(cxt, "srcpath '%s'", src));
+	DBG(CXT, ul_debugobj(cxt, "srcpath '%s'", src));
 
 	cache = mnt_context_get_cache(cxt);
 
@@ -1427,7 +1488,7 @@ int mnt_context_prepare_srcpath(struct libmnt_context *cxt)
 	 }
 
 	if (rc) {
-		DBG(CXT, mnt_debug_h(cxt, "failed to prepare srcpath [rc=%d]", rc));
+		DBG(CXT, ul_debugobj(cxt, "failed to prepare srcpath [rc=%d]", rc));
 		return rc;
 	}
 
@@ -1436,7 +1497,7 @@ int mnt_context_prepare_srcpath(struct libmnt_context *cxt)
 
 	if ((cxt->mountflags & (MS_BIND | MS_MOVE | MS_REMOUNT))
 	    || mnt_fs_is_pseudofs(cxt->fs)) {
-		DBG(CXT, mnt_debug_h(cxt, "REMOUNT/BIND/MOVE/pseudo FS source: %s", path));
+		DBG(CXT, ul_debugobj(cxt, "REMOUNT/BIND/MOVE/pseudo FS source: %s", path));
 		return rc;
 	}
 
@@ -1449,7 +1510,7 @@ int mnt_context_prepare_srcpath(struct libmnt_context *cxt)
 			return rc;
 	}
 
-	DBG(CXT, mnt_debug_h(cxt, "final srcpath '%s'",
+	DBG(CXT, ul_debugobj(cxt, "final srcpath '%s'",
 				mnt_fs_get_source(cxt->fs)));
 	return 0;
 }
@@ -1478,7 +1539,7 @@ static int mkdir_target(const char *tgt, struct libmnt_fs *fs)
 		mode = strtol(mstr, &end, 8);
 
 		if (errno || !end || mstr + mstr_sz != end) {
-			DBG(CXT, mnt_debug("failed to parse mkdir mode '%s'", mstr));
+			DBG(CXT, ul_debug("failed to parse mkdir mode '%s'", mstr));
 			return -MNT_ERR_MOUNTOPT;
 		}
 	}
@@ -1490,7 +1551,7 @@ static int mkdir_target(const char *tgt, struct libmnt_fs *fs)
 
 	rc = mkdir_p(tgt, mode);
 	if (rc)
-		DBG(CXT, mnt_debug("mkdir %s failed: %m", tgt));
+		DBG(CXT, ul_debug("mkdir %s failed: %m", tgt));
 
 	return rc;
 }
@@ -1508,7 +1569,7 @@ int mnt_context_prepare_target(struct libmnt_context *cxt)
 	if (!cxt || !cxt->fs)
 		return -EINVAL;
 
-	DBG(CXT, mnt_debug_h(cxt, "preparing target path"));
+	DBG(CXT, ul_debugobj(cxt, "preparing target path"));
 
 	tgt = mnt_fs_get_target(cxt->fs);
 	if (!tgt)
@@ -1533,13 +1594,47 @@ int mnt_context_prepare_target(struct libmnt_context *cxt)
 	}
 
 	if (rc)
-		DBG(CXT, mnt_debug_h(cxt, "failed to prepare target '%s'", tgt));
+		DBG(CXT, ul_debugobj(cxt, "failed to prepare target '%s'", tgt));
 	else
-		DBG(CXT, mnt_debug_h(cxt, "final target '%s'",
+		DBG(CXT, ul_debugobj(cxt, "final target '%s'",
 					mnt_fs_get_target(cxt->fs)));
 	return 0;
 }
 
+/* Guess type, but not set to cxt->fs, use free() for the result. It's no error
+ * when we're not able to guess a filesystem type.
+ */
+int mnt_context_guess_srcpath_fstype(struct libmnt_context *cxt, char **type)
+{
+	int rc = 0;
+	const char *dev = mnt_fs_get_srcpath(cxt->fs);
+
+	*type = NULL;
+
+	if (!dev)
+		goto done;
+
+	if (access(dev, F_OK) == 0) {
+		struct libmnt_cache *cache = mnt_context_get_cache(cxt);
+		int ambi = 0;
+
+		*type = mnt_get_fstype(dev, &ambi, cache);
+		if (cache && *type)
+			*type = strdup(*type);
+		if (ambi)
+			rc = -MNT_ERR_AMBIFS;
+	} else {
+		DBG(CXT, ul_debugobj(cxt, "access(%s) failed [%m]", dev));
+		if (strchr(dev, ':') != NULL)
+			*type = strdup("nfs");
+		else if (!strncmp(dev, "//", 2))
+			*type = strdup("cifs");
+	}
+
+done:
+	return rc;
+}
+
 /*
  * It's usually no error when we're not able to detect the filesystem type -- we
  * will try to use the types from /{etc,proc}/filesystems.
@@ -1547,7 +1642,6 @@ int mnt_context_prepare_target(struct libmnt_context *cxt)
 int mnt_context_guess_fstype(struct libmnt_context *cxt)
 {
 	char *type;
-	const char *dev;
 	int rc = 0;
 
 	assert(cxt);
@@ -1574,32 +1668,13 @@ int mnt_context_guess_fstype(struct libmnt_context *cxt)
 	if (cxt->fstype_pattern)
 		goto done;
 
-	dev = mnt_fs_get_srcpath(cxt->fs);
-	if (!dev)
-		goto done;
-
-	if (access(dev, F_OK) == 0) {
-		struct libmnt_cache *cache = mnt_context_get_cache(cxt);
-		int ambi = 0;
-
-		type = mnt_get_fstype(dev, &ambi, cache);
-		if (type) {
-			rc = mnt_fs_set_fstype(cxt->fs, type);
-			if (!cache)
-				free(type);	/* type is not cached */
-		}
-		if (ambi)
-			rc = -MNT_ERR_AMBIFS;
-	} else {
-		DBG(CXT, mnt_debug_h(cxt, "access(%s) failed [%m]", dev));
-		if (strchr(dev, ':') != NULL)
-			rc = mnt_fs_set_fstype(cxt->fs, "nfs");
-		else if (!strncmp(dev, "//", 2))
-			rc = mnt_fs_set_fstype(cxt->fs, "cifs");
-	}
-
+	rc = mnt_context_guess_srcpath_fstype(cxt, &type);
+	if (rc == 0 && type)
+		__mnt_fs_set_fstype_ptr(cxt->fs, type);
+	else
+		free(type);
 done:
-	DBG(CXT, mnt_debug_h(cxt, "FS type: %s [rc=%d]",
+	DBG(CXT, ul_debugobj(cxt, "FS type: %s [rc=%d]",
 				mnt_fs_get_fstype(cxt->fs), rc));
 	return rc;
 none:
@@ -1658,7 +1733,7 @@ int mnt_context_prepare_helper(struct libmnt_context *cxt, const char *name,
 			rc = stat(helper, &st);
 		}
 
-		DBG(CXT, mnt_debug_h(cxt, "%-25s ... %s", helper,
+		DBG(CXT, ul_debugobj(cxt, "%-25s ... %s", helper,
 					rc ? "not found" : "found"));
 		if (rc)
 			continue;
@@ -1680,7 +1755,7 @@ int mnt_context_merge_mflags(struct libmnt_context *cxt)
 
 	assert(cxt);
 
-	DBG(CXT, mnt_debug_h(cxt, "merging mount flags"));
+	DBG(CXT, ul_debugobj(cxt, "merging mount flags"));
 
 	rc = mnt_context_get_mflags(cxt, &fl);
 	if (rc)
@@ -1693,7 +1768,7 @@ int mnt_context_merge_mflags(struct libmnt_context *cxt)
 		return rc;
 	cxt->user_mountflags = fl;
 
-	DBG(CXT, mnt_debug_h(cxt, "final flags: VFS=%08lx user=%08lx",
+	DBG(CXT, ul_debugobj(cxt, "final flags: VFS=%08lx user=%08lx",
 			cxt->mountflags, cxt->user_mountflags));
 
 	cxt->flags |= MNT_FL_MOUNTFLAGS_MERGED;
@@ -1713,10 +1788,10 @@ int mnt_context_prepare_update(struct libmnt_context *cxt)
 	assert(cxt->action);
 	assert((cxt->flags & MNT_FL_MOUNTFLAGS_MERGED));
 
-	DBG(CXT, mnt_debug_h(cxt, "prepare update"));
+	DBG(CXT, ul_debugobj(cxt, "prepare update"));
 
 	if (mnt_context_propagation_only(cxt)) {
-		DBG(CXT, mnt_debug_h(cxt, "skip update: only MS_PROPAGATION"));
+		DBG(CXT, ul_debugobj(cxt, "skip update: only MS_PROPAGATION"));
 		return 0;
 	}
 
@@ -1727,30 +1802,26 @@ int mnt_context_prepare_update(struct libmnt_context *cxt)
 		mnt_context_disable_mtab(cxt, TRUE);
 
 	if (mnt_context_is_nomtab(cxt)) {
-		DBG(CXT, mnt_debug_h(cxt, "skip update: NOMTAB flag"));
+		DBG(CXT, ul_debugobj(cxt, "skip update: NOMTAB flag"));
 		return 0;
 	}
-	if (cxt->helper) {
-		DBG(CXT, mnt_debug_h(cxt, "skip update: external helper"));
-		return 0;
-	}
-	if (!cxt->mtab_writable && !cxt->utab_writable) {
-		DBG(CXT, mnt_debug_h(cxt, "skip update: no writable destination"));
+	if (!mnt_context_get_writable_tabpath(cxt)) {
+		DBG(CXT, ul_debugobj(cxt, "skip update: no writable destination"));
 		return 0;
 	}
 	/* 0 = success, 1 = not called yet */
 	if (cxt->syscall_status != 1 && cxt->syscall_status != 0) {
-		DBG(CXT, mnt_debug_h(cxt,
+		DBG(CXT, ul_debugobj(cxt,
 				"skip update: syscall failed [status=%d]",
 				cxt->syscall_status));
 		return 0;
 	}
 
 	if (!cxt->update) {
-		const char *name = cxt->mtab_writable ? cxt->mtab_path : cxt->utab_path;
+		const char *name = mnt_context_get_writable_tabpath(cxt);
 
 		if (cxt->action == MNT_ACT_UMOUNT && is_file_empty(name)) {
-			DBG(CXT, mnt_debug_h(cxt,
+			DBG(CXT, ul_debugobj(cxt,
 				"skip update: umount, no table"));
 			return 0;
 		}
@@ -1759,7 +1830,8 @@ int mnt_context_prepare_update(struct libmnt_context *cxt)
 		if (!cxt->update)
 			return -ENOMEM;
 
-		mnt_update_set_filename(cxt->update, name, !cxt->mtab_writable);
+		mnt_update_set_filename(cxt->update, name,
+				!mnt_context_mtab_writable(cxt));
 	}
 
 	if (cxt->action == MNT_ACT_UMOUNT)
@@ -1779,19 +1851,33 @@ int mnt_context_update_tabs(struct libmnt_context *cxt)
 	assert(cxt);
 
 	if (mnt_context_is_nomtab(cxt)) {
-		DBG(CXT, mnt_debug_h(cxt, "don't update: NOMTAB flag"));
-		return 0;
-	}
-	if (cxt->helper) {
-		DBG(CXT, mnt_debug_h(cxt, "don't update: external helper"));
+		DBG(CXT, ul_debugobj(cxt, "don't update: NOMTAB flag"));
 		return 0;
 	}
 	if (!cxt->update || !mnt_update_is_ready(cxt->update)) {
-		DBG(CXT, mnt_debug_h(cxt, "don't update: no update prepared"));
+		DBG(CXT, ul_debugobj(cxt, "don't update: no update prepared"));
 		return 0;
 	}
-	if (cxt->syscall_status) {
-		DBG(CXT, mnt_debug_h(cxt, "don't update: syscall failed/not called"));
+
+	/* check utab update when external helper executed */
+	if (mnt_context_helper_executed(cxt)
+	    && mnt_context_get_helper_status(cxt) == 0
+	    && mnt_context_utab_writable(cxt)) {
+
+		if (mnt_update_already_done(cxt->update, cxt->lock)) {
+			DBG(CXT, ul_debugobj(cxt, "don't update: error evaluate or already updated"));
+			return 0;
+		}
+	} else if (cxt->helper) {
+		DBG(CXT, ul_debugobj(cxt, "don't update: external helper"));
+		return 0;
+	}
+
+	if (cxt->syscall_status != 0
+	    && !(mnt_context_helper_executed(cxt) &&
+		 mnt_context_get_helper_status(cxt) == 0)) {
+
+		DBG(CXT, ul_debugobj(cxt, "don't update: syscall/helper failed/not called"));
 		return 0;
 	}
 
@@ -1849,7 +1935,7 @@ static int apply_table(struct libmnt_context *cxt, struct libmnt_table *tb,
 	if (!fs)
 		return -MNT_ERR_NOFSTAB;	/* not found */
 
-	DBG(CXT, mnt_debug_h(cxt, "apply entry:"));
+	DBG(CXT, ul_debugobj(cxt, "apply entry:"));
 	DBG(CXT, mnt_fs_print_debug(fs, stderr));
 
 	/* copy from tab to our FS description
@@ -1904,10 +1990,10 @@ int mnt_context_apply_fstab(struct libmnt_context *cxt)
 		return 0;
 
 	if (mnt_context_is_restricted(cxt)) {
-		DBG(CXT, mnt_debug_h(cxt, "force fstab usage for non-root users!"));
+		DBG(CXT, ul_debugobj(cxt, "force fstab usage for non-root users!"));
 		cxt->optsmode = MNT_OMODE_USER;
 	} else if (cxt->optsmode == 0) {
-		DBG(CXT, mnt_debug_h(cxt, "use default optsmode"));
+		DBG(CXT, ul_debugobj(cxt, "use default optsmode"));
 		cxt->optsmode = MNT_OMODE_AUTO;
 	} else if (cxt->optsmode & MNT_OMODE_NOTAB) {
 		cxt->optsmode &= ~MNT_OMODE_FSTAB;
@@ -1920,7 +2006,7 @@ int mnt_context_apply_fstab(struct libmnt_context *cxt)
 		tgt = mnt_fs_get_target(cxt->fs);
 	}
 
-	DBG(CXT, mnt_debug_h(cxt, "OPTSMODE: ignore=%d, append=%d, prepend=%d, "
+	DBG(CXT, ul_debugobj(cxt, "OPTSMODE: ignore=%d, append=%d, prepend=%d, "
 				  "replace=%d, force=%d, fstab=%d, mtab=%d",
 				  cxt->optsmode & MNT_OMODE_IGNORE ? 1 : 0,
 				  cxt->optsmode & MNT_OMODE_APPEND ? 1 : 0,
@@ -1932,19 +2018,19 @@ int mnt_context_apply_fstab(struct libmnt_context *cxt)
 
 	/* fstab is not required if source and target are specified */
 	if (src && tgt && !(cxt->optsmode & MNT_OMODE_FORCE)) {
-		DBG(CXT, mnt_debug_h(cxt, "fstab not required -- skip"));
+		DBG(CXT, ul_debugobj(cxt, "fstab not required -- skip"));
 		return 0;
 	}
 
 	if (!src && tgt
 	    && !(cxt->optsmode & MNT_OMODE_FSTAB)
 	    && !(cxt->optsmode & MNT_OMODE_MTAB)) {
-		DBG(CXT, mnt_debug_h(cxt, "only target; fstab/mtab not required "
+		DBG(CXT, ul_debugobj(cxt, "only target; fstab/mtab not required "
 					  "-- skip, probably MS_PROPAGATION"));
 		return 0;
 	}
 
-	DBG(CXT, mnt_debug_h(cxt,
+	DBG(CXT, ul_debugobj(cxt,
 		"trying to apply fstab (src=%s, target=%s)", src, tgt));
 
 	/* let's initialize cxt->fs */
@@ -1959,13 +2045,19 @@ int mnt_context_apply_fstab(struct libmnt_context *cxt)
 
 	/* try mtab */
 	if (rc < 0 && (cxt->optsmode & MNT_OMODE_MTAB)) {
-		DBG(CXT, mnt_debug_h(cxt, "trying to apply from mtab"));
+		DBG(CXT, ul_debugobj(cxt, "trying to apply from mtab"));
 		rc = mnt_context_get_mtab(cxt, &tab);
 		if (!rc)
 			rc = apply_table(cxt, tab, MNT_ITER_BACKWARD);
 	}
-	if (rc)
-		DBG(CXT, mnt_debug_h(cxt, "failed to find entry in fstab/mtab"));
+	if (rc) {
+		DBG(CXT, ul_debugobj(cxt, "failed to find entry in fstab/mtab [rc=%d]: %m", rc));
+
+		/* force to "not found in fstab/mtab" error, the details why
+		 * not found are not so important and may be misinterpreted by
+		 * applications... */
+		rc = -MNT_ERR_NOFSTAB;
+	}
 	return rc;
 }
 
@@ -2095,7 +2187,7 @@ int mnt_context_set_syscall_status(struct libmnt_context *cxt, int status)
 	if (!cxt)
 		return -EINVAL;
 
-	DBG(CXT, mnt_debug_h(cxt, "syscall status set to: %d", status));
+	DBG(CXT, ul_debugobj(cxt, "syscall status set to: %d", status));
 	cxt->syscall_status = status;
 	return 0;
 }
@@ -2148,7 +2240,7 @@ int mnt_context_init_helper(struct libmnt_context *cxt, int action,
 	if (!rc)
 		cxt->action = action;
 
-	DBG(CXT, mnt_debug_h(cxt, "initialized for [u]mount.<type> helper [rc=%d]", rc));
+	DBG(CXT, ul_debugobj(cxt, "initialized for [u]mount.<type> helper [rc=%d]", rc));
 	return rc;
 }
 
@@ -2217,7 +2309,7 @@ static int mnt_context_add_child(struct libmnt_context *cxt, pid_t pid)
 	if (!pids)
 		return -ENOMEM;
 
-	DBG(CXT, mnt_debug_h(cxt, "add new child %d", pid));
+	DBG(CXT, ul_debugobj(cxt, "add new child %d", pid));
 	cxt->children = pids;
 	cxt->children[cxt->nchildren++] = pid;
 
@@ -2233,7 +2325,7 @@ int mnt_fork_context(struct libmnt_context *cxt)
 	if (!mnt_context_is_parent(cxt))
 		return -EINVAL;
 
-	DBG(CXT, mnt_debug_h(cxt, "forking context"));
+	DBG(CXT, ul_debugobj(cxt, "forking context"));
 
 	DBG_FLUSH;
 
@@ -2241,13 +2333,13 @@ int mnt_fork_context(struct libmnt_context *cxt)
 
 	switch (pid) {
 	case -1: /* error */
-		DBG(CXT, mnt_debug_h(cxt, "fork failed %m"));
+		DBG(CXT, ul_debugobj(cxt, "fork failed %m"));
 		return -errno;
 
 	case 0: /* child */
 		cxt->pid = getpid();
 		mnt_context_enable_fork(cxt, FALSE);
-		DBG(CXT, mnt_debug_h(cxt, "child created"));
+		DBG(CXT, ul_debugobj(cxt, "child created"));
 		break;
 
 	default:
@@ -2276,7 +2368,7 @@ int mnt_context_wait_for_children(struct libmnt_context *cxt,
 		if (!pid)
 			continue;
 		do {
-			DBG(CXT, mnt_debug_h(cxt,
+			DBG(CXT, ul_debugobj(cxt,
 					"waiting for child (%d/%d): %d",
 					i + 1, cxt->nchildren, pid));
 			errno = 0;
